package asw.asw850.asynchronous.client;

import asw.asw850.asynchronous.*;  

import javax.ejb.EJB; 

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.logging.*;

import asw.util.logging.*;
import asw.util.sleep.Sleeper;

/* 
 * Application client per l'EJB Hello. 
 */

public class Main {

		@EJB(lookup = "ejb/asw850/AsynchronousBean")
		public static AsynchronousRemote asynch; 
		
		/* logger */ 
		private static Logger logger = AswLogger.getInstance().getLogger("asw.asw850.asynchronous.client"); 

		public static void main(String[] args) {
			new Main() . run(); 
		}
		
		public Main() {		
		}
		
		private void run() {
	    	/* usa il servizio Asynchronous */ 
	    	logger.info("Client: Ora uso il servizio Asynchronous");

	    	/* dovrebbe restituire "LUCA" */ 
	    	callAsynchOperation("Luca"); 

	    	/* l'operazione dovrebbe venire cancellata */ 
	    	callAsynchOperation("Benedetta"); 

	    	/* dovrebbe sollevare un'eccezione di business */ 
	    	callAsynchOperation(null); 

	    	logger.info("Client: Ho finito di usare il servizio Asynchronous");

		}

	    /* chiama l'operazione asincrona */ 
	    private void callAsynchOperation(String param) {
	    	/* usa il servizio Asynchronous */ 
			logger.info("Client: calling asynchOperation(" + param + ")"); 
			Future<String> futureResult;
			long startingTime = System.currentTimeMillis();
			try {
				futureResult = asynch.asynchOperation(param);
				/* aspetta, ma al massimo un paio di secondi, facendo altre cose */ 
				for (int i=0; i<10; i++) {
					long currentTime = System.currentTimeMillis();
					logger.info("Client: asynchOperation(" + param + "): " + 
							"in attesa da " + (currentTime-startingTime) + " ms)"); 
					Sleeper.sleep(250);
					if (futureResult.isDone()) {
						break;
					}
				}
				/* se non ha ancora finito, annulla l'operazione */
				if (!futureResult.isDone()) {
					logger.info("Client: asynchOperation(" + param + "): cancellazione"); 
					futureResult.cancel(true);
				}
				/* attende ancora la terminazione */ 
				while (true) {
					long currentTime = System.currentTimeMillis();
					logger.info("Client: asynchOperation(" + param + ") annullata: " + 
							"in attesa da " + (currentTime-startingTime) + " ms)"); 
					Sleeper.sleep(250);
					if (futureResult.isDone() || futureResult.isCancelled()) {
						break;
					}
				}
				if (futureResult.isCancelled()) {
					long endingTime = System.currentTimeMillis();
					logger.info("Client: asynchOperation(" + param + ") ==> was canceled" +  
							" (" + (endingTime-startingTime) + " ms)");					
				} else {
					String result = futureResult.get(); 
					long endingTime = System.currentTimeMillis();
					logger.info("Client: asynchOperation(" + param + ") ==> " + result + 
								" (" + (endingTime-startingTime) + " ms)");
				}
			} catch (AsynchronousException | InterruptedException | ExecutionException e) {
//				e.printStackTrace();
				logger.info("Client: asynchOperation(" + param + ") ha sollevato un'eccezione: " + e.getMessage()); 				
			}
	    }

}