package asw.asw850.asynchronous;

import java.util.concurrent.Future;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.AsyncResult;
import javax.ejb.Asynchronous;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.interceptor.InvocationContext;

import asw.util.sleep.Sleeper;

/**
 * Session Bean implementation class AsynchronousBean
 */
@Stateless(mappedName = "ejb/asw850/AsynchronousBean")
public class AsynchronousBean implements AsynchronousRemote {

	/* logger */ 
	private Logger logger; 

	/* il contesto  necessario per la cancellazione 
	 * delle operazioni asincrone */
    @Resource
    SessionContext ctx;
    
	/* inizializza un'istanza del bean */ 
	@PostConstruct
	public void initialize() {
		/* fa il logging sul log del server EJB */ 
    	logger = Logger.getLogger("asw.asw850.asynchronous"); 
		logger.info("AsynchronousBean.initialize(): initializing Asynchronous instance");
	}
	
	/** Un'operazione asincrona (che pu essere interrotta). */ 
	@Asynchronous
	public Future<String> asynchOperation(String param) throws AsynchronousException, InterruptedException {
		String result = null;
		logger.info("Asynchronous.asynchOperation(" + param + ") called");
		
		if (param==null) {
			throw new AsynchronousException("Invalid parameters for asynchOperation()");
		}
		/* diciamo che questa operazione perde un po' di tempo e 
		 * poi restituisce proprio la stringa param */
		for (int i=0; i<param.length()*10; i++) {
			Sleeper.sleep(40); 
			if (ctx.wasCancelCalled()) {
				logger.info("Asynchronous.asynchOperation(" + param + ") was cancelled");
				/* credo che la cosa giusta da fare qui sia sollevare un'eccezione */ 
				throw new InterruptedException("Operation asynchOperation() was cancelled");
			}
		}
		result = param.toUpperCase(); 
		logger.info("Asynchronous.asynchOperation(" + param + ") ==> " + result);
		return new AsyncResult<String>(result);
	}

}
