package asw.asw850.consumatore;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;

import java.util.logging.Logger;
import asw.util.logging.AswLogger;

/*
 * Ascoltatore di messaggi. 
 * Implementa MessageListener. 
 * Visualizza il messaggio ricevuto. 
 */
public class ConsumatoreMessageProcessor implements MessageListener {
	
	/* logger */ 
	private static Logger logger = AswLogger.getInstance().getLogger("asw.asw850.consumatore"); 
	
	/* numero di messaggi ricevuti */ 
	private int messaggiRicevuti; 
	
	/**
	 * Crea un nuovo ConsumatoreMessageProcessor. 
	 */
	public ConsumatoreMessageProcessor() {
		this.messaggiRicevuti = 0; 
	}
	
    /**
     * Gestisce la ricezione di un messaggio JMS m
     * (implementa MessageListener).
     */
    public synchronized void onMessage(Message m) {
    	/* ricevuto un messaggio, ne delega l'elaborazione
    	 * al metodo processMessage */
//		logger.fine("MessageListener.onMessage()");
    	if (m instanceof TextMessage) {
    		TextMessage  message = (TextMessage) m;
    		try {
    			/* gestisce la ricezione del messaggio message */
    			String text = message.getText();
    	    	/* delega la vera gestione del messaggio 
    	    	 * al metodo processMessage */
    	    	this.processMessage(text);
    		} catch (JMSException e) {
    			logger.info("MessageListener.onMessage(): JMSException: " + e.toString());
    		}
    	}
    }
	
	/* 
	 * Gestisce la ricezione di un messaggio: 
	 * visualizza il messaggio ricevuto. 
	 */ 
	private void processMessage(String message) {
		/* conta il messaggio ricevuto */ 
		this.messaggiRicevuti++; 
		/* visualizza il messaggio */ 
		logger.info("MessageListener: Ricevuto messaggio # " + messaggiRicevuti + ": "+ message); 
	}

}
