/*
 * Main.java
 *
 * Classe principale per l'application client SimpleAsynchProducer.
 * 
 * Esecuzione: dal prompt dei comandi, dalla cartella del progetto, tramite appclient: 
 * appclient -client build\SimpleAsynchConsumer <nomeConsumer> <destinazione (queue o topic)> <#messaggi> <ritardo tra messaggi>
 *
 * Variante di un esempio nel tutorial per Java EE.
 *
 * @author cabibbo
 */

package asw.asw850.consumatore;

import java.util.logging.Logger;

import javax.jms.*;

import asw.jms.simpleasynchconsumer.SimpleAsynchConsumer;
import asw.jms.simpleasynchconsumer.SimpleAsynchConsumerThread;
import asw.util.cancellable.KeyboardKiller;
import asw.util.logging.AswLogger;

public class ConsumatoreMessaggi {

	/* logger */ 
	private static Logger logger = AswLogger.getInstance().getLogger("asw.asw850.consumatore"); 

	/* la destinazione da cui consumare messaggi */ 
	private Destination destination; 

	/* connection factory */ 
	private ConnectionFactory connectionFactory; 
	
	/* Crea un nuovo consumatore di messaggi. */	
	public ConsumatoreMessaggi(Destination destination, ConnectionFactory connectionFactory) {
		this.destination = destination;
		this.connectionFactory = connectionFactory;
	}

    public void run() {
    	
		/* definizione dei parametri */
    	String name = "Consumatore[" + (int)(Math.random()*1000) + "]";

		/* crea il consumatore "finale" del messaggio */ 
		MessageListener listener = new ConsumatoreMessageProcessor(); 

		/* crea un consumatore sulla propria destinazione - girer messaggi al proprio processore/listener */ 
		SimpleAsynchConsumer consumer= new SimpleAsynchConsumer(name, destination, connectionFactory, listener);
        logger.info("Creato consumatore: " + consumer.toString());

        /* per interrompere il consumatore premendo INVIO */ 
        KeyboardKiller k = new KeyboardKiller(); 
        k.add(consumer); 
        k.start(); 

        /* avvia il consumatore (entro un thread) */ 
        SimpleAsynchConsumerThread tc = new SimpleAsynchConsumerThread(consumer);
        tc.start(); 

    	/* attende la terminazione del thread - poi termina il programma */ 
        try {
			tc.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} 

        /* disconnette il consumatore */ 
        consumer.disconnect(); 
        /* termina */ 
        System.exit(0);
    }
    
}
